@@ -3,19 +3,20 @@
    Copyright (C) 2002-2007 Bas Wijnen <wijnen@debian.org>
    Copyright (C) 2005 Jan Wilmans <jw@dds.nl>
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   This file is part of z80asm.
 
-   This program is distributed in the hope that it will be useful,
+   Z80asm is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+
+   Z80asm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation, Inc.,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "z80asm.h"
@@ -618,6 +619,7 @@ write_one_byte (int b, int list)
       listdepth += 3;
     }
   addr++;
+  addr &= 0xffff;
 }
 
 /* write byte to outfile and possibly some index things as well */
@@ -736,17 +738,42 @@ new_reference (const char *p, int type, char delimiter, int ds_count)
   else
     {
       /* the expression is not valid (yet), we need to make a real reference.  */
-      if (NULL == (tmp = malloc (sizeof (struct reference) + strlen (p))))
+      tmp = malloc (sizeof (struct reference) + strlen (p));
+      if (!tmp)
 	{
 	  printerr (1, "unable to allocate memory for reference %s\n", p);
 	  return;
 	}
+      tmp->file = malloc (strlen (stack[sp].name) + 1);
+      if (!tmp->file)
+	{
+	  printerr (1, "unable to allocate memory for reference filename\n");
+	  free (tmp);
+	  return;
+	}
+      strcpy (tmp->file, stack[sp].name);
+      if (stack[sp].dir)
+	{
+	  tmp->dir = malloc (strlen (stack[sp].dir->name)
+			  + sizeof (struct includedir));
+	  if (!tmp->dir)
+	    {
+	      printerr (1, "unable to allocate memory for reference dir\n");
+	      free (tmp->file);
+	      free (tmp);
+	      return;
+	    }
+	  strcpy (tmp->dir->name, stack[sp].dir->name);
+	}
+      else
+	tmp->dir = NULL;
       opos = ftell (outfile);
       lpos = havelist ? ftell (listfile) : 0;
       if (verbose >= 3)
 	fprintf (stderr, "%5d (0x%04x): reference set to %s (delimiter=%c, "
 		 "sp=%d)\n", stack[sp].line, addr, p, delimiter, sp);
       strcpy (tmp->input, p);
+      tmp->line = stack[sp].line;
       tmp->addr = addr;
       tmp->baseaddr = baseaddr;
       tmp->count = ds_count;
@@ -1397,7 +1424,7 @@ wrt_ref (int val, int type, int count)
       return;
     case TYPE_RELB:
       val -= count;
-      if (val < -128 || val > 127)
+      if (val & 0xff80 && ~val & 0xff80) 
 	{
 	  printerr (1, "relative jump out of range (%d)\n", val);
 	}
@@ -2464,7 +2491,7 @@ assemble (void)
 	      file_ended = 1;
 	      break;
 	    case ORG:
-	      addr = rd_expr (&ptr, '\0', NULL, sp, 1);
+	      addr = rd_expr (&ptr, '\0', NULL, sp, 1) & 0xffff;
 	      break;
 	    case INCLUDE:
 	      if (sp + 1 >= MAX_INCLUDE)
@@ -2542,6 +2569,7 @@ assemble (void)
 			break;
 		      }
 		    addr += num;
+		    addr &= 0xffff;
 		  }
 		fclose (incfile);
 		free (name);
@@ -2705,6 +2733,8 @@ assemble (void)
   {
     struct reference *next;
     struct reference *tmp;
+    /* Add a stack frame for error reporting.  */
+    ++sp;
     for (tmp = firstreference; tmp; tmp = next)
       {
 	int ref;
@@ -2712,8 +2742,14 @@ assemble (void)
 	fseek (outfile, tmp->oseekpos, SEEK_SET);
 	if (havelist)
 	  fseek (listfile, tmp->lseekpos, SEEK_SET);
+	stack[sp].name = tmp->file;
+	stack[sp].dir = tmp->dir;
+	stack[sp].line = tmp->line;
 	ref = compute_ref (tmp, 0);
 	wrt_ref (ref, tmp->type, tmp->count);
+	if (tmp->dir)
+	  free (tmp->dir);
+	free (tmp->file);
 	free (tmp);
       }
   }
